-- 来源：EUI
-- 作者：EUI
-- 链接：
-- 修改：L
local LUI, T, E, L, V, P, G = unpack(select(2, ...))
if LUI:CheckDB("modules", "misc", "general", "rightButtonMenu") == true then return end

local RBM = E:NewModule("rightButtonMenu")

local locale = T.GetLocale()
local CHAT_BLACKLIST_INTRO = ""
if locale == "zhCN" then
	CHAT_BLACKLIST_INTRO = " 已被加入信息过滤黑名单!"
elseif locale == "zhTW" then
	CHAT_BLACKLIST_INTRO = " 已被加入資訊過濾黑名單!"
else
	CHAT_BLACKLIST_INTRO = " Has been added to the information filtering blacklist!"
end

local function urlencode(s)
	s = T.gsub(s, "([^%w%.%- ])", function(c)
			return T.string_format("%%%02X", T.string_byte(c))
		end)
	return T.gsub(s, " ", "+")
end

local function gethost()
	local host = "http://us.battle.net/wow/en/character/"
	if (T.GetLocale() == "zhTW") then
		host = "http://tw.battle.net/wow/zh/character/"
	elseif (T.GetLocale() == "zhCN") then
		host = "http://www.battlenet.com.cn/wow/zh/character/"
	end
	return host
end

local function getfullname()
	local unit = UIDROPDOWNMENU_INIT_MENU.unit
	local name = UIDROPDOWNMENU_INIT_MENU.name
	local server = UIDROPDOWNMENU_INIT_MENU.server

	if (server and (not unit or T.UnitRealmRelationship(unit) ~= LE_REALM_RELATION_SAME)) then
		return name .. "-" .. server
	else
		return name
	end
end

RBM.friend_features = {
	"ARMORY",
	"NAME_COPY",
	"SEND_WHO",
	"FRIEND_ADD",
	"GUILD_ADD",
	"MYSTATS",
	"LUIBLACKPLAYER",
}
RBM.cr_features = {
	"NAME_COPY",
	"SEND_WHO",
	"FRIEND_ADD",
	"LUIBLACKPLAYER",
}
RBM.guild_features = {
	"ARMORY",
	"NAME_COPY",
	"FRIEND_ADD",
	"LUIBLACKPLAYER",
}
RBM.player_features = {
	"ARMORY",
	"NAME_COPY",
	"LUIBLACKPLAYER",
}

RBM.UnitPopupButtonsExtra = {
	["ARMORY"] = true,
	["NAME_COPY"] = true,
	["SEND_WHO"] = true,
	["GUILD_ADD"] = true,
	["FRIEND_ADD"] = true,
	["MYSTATS"] = true,
	["LUIBLACKPLAYER"] = true,
}

RBM.dropdownmenu_show = {
	["SELF"] = true,
	["PLAYER"] = true,
	["PARTY"] = true,
	["RAID_PLAYER"] = true,
	["TARGET"] = true,
}

UnitPopupButtons["LUIRB"] = {
	text = "增强功能",
	isTitle = true,
	isUninteractable = true,
	isSubsection = true,
	isSubsectionTitle = true,
	isSubsectionSeparator = true,
}

UnitPopupButtons["ARMORY"] = {
	text = L["Armory"],
	func = function()
		local name = UIDROPDOWNMENU_INIT_MENU.name
		local server = UIDROPDOWNMENU_INIT_MENU.server

		if name then
			local armory = gethost() .. urlencode(server or T.GetRealmName()) .. "/" .. urlencode(name) .. "/advanced"
			local editBox = ChatEdit_ChooseBoxForSend()
			ChatEdit_ActivateChat(editBox)
			editBox:SetText(armory)
			editBox:HighlightText()
		end
	end
}


UnitPopupButtons["SEND_WHO"] = {
	text = L["Query Detail"],
	func = function()
		local fullname = getfullname()
		if fullname then
			T.SendWho(fullname)
		end
	end
}

UnitPopupButtons["NAME_COPY"] = {
	text = L["Get Name"],
	func = function()
		local fullname = getfullname()
		if fullname then
			local editBox = ChatEdit_ChooseBoxForSend()
			ChatEdit_ActivateChat(editBox)
			editBox:Insert(fullname)
			editBox:HighlightText()
		end
	end
}

UnitPopupButtons["GUILD_ADD"] = {
	text = L["Guild Invite"],
	func = function()
		local fullname = getfullname()
		if fullname then
			T.GuildInvite(fullname)
		end
	end
}

UnitPopupButtons["FRIEND_ADD"] = {
	text = L["Add Friend"],
	func = function()
		local fullname = getfullname()
		if fullname then
			T.C_FriendList_AddFriend(fullname)
		end
	end
}

UnitPopupButtons["MYSTATS"] = {
	text = L["Report MyStats"],
	func = function()
		local fullname = getfullname()

		if fullname then
			local CRITICAL = TEXT_MODE_A_STRING_RESULT_CRITICAL or STAT_CRITICAL_STRIKE
			CRITICAL = T.gsub(CRITICAL, "[()]","")
			T.SendChatMessage(T.string_format("%s:%.1f %s:%s", ITEM_LEVEL_ABBR, select(2,T.GetAverageItemLevel()), HP, T.AbbreviateNumbers(T.UnitHealthMax("player"))), "WHISPER", nil, fullname)
			T.SendChatMessage(T.string_format(" - %s:%.2f%%", STAT_HASTE, T.GetHaste()), "WHISPER", nil, fullname)
			T.SendChatMessage(T.string_format(" - %s:%.2f%%", STAT_MASTERY, T.GetMasteryEffect()), "WHISPER", nil, fullname)
			T.SendChatMessage(T.string_format(" - %s:%.2f%%", CRITICAL, T.max(T.GetRangedCritChance(), T.GetCritChance(), T.GetSpellCritChance(2))), "WHISPER", nil, fullname)
			T.SendChatMessage(T.string_format(" - %s:%.2f%%", STAT_VERSATILITY, T.GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE) + T.GetVersatilityBonus(CR_VERSATILITY_DAMAGE_DONE)), "WHISPER", nil, fullname)
		end
	end
}

UnitPopupButtons["LUIBLACKPLAYER"] = {
	text = L["LUIBLACKPLAYER"],
	func = function()
		if not ElvUI then 
			LUI:Print("Not find ElvUI")
			return
		end
		local fullname = getfullname()
		if fullname then
			E.global.lui.modules.filters.playerFilter.nameFilter[fullname] = true
			LUI:Print(fullname..CHAT_BLACKLIST_INTRO)
			if T.IsAddOnLoaded("ElvUI_Config") then
				for k, v in T.pairs(E.global.lui.modules.filters.playerFilter.nameFilter) do
					E.Options.args.lui.args.modules.args.filters.args.playerFilter.args.nameFilter.args.playerBlacklist.values[k] = k
				end
			end
		end
	end
}

function RBM:Initialize()
	T.table_insert(UnitPopupMenus["FRIEND"], #UnitPopupMenus["FRIEND"] - 1, "LUIRB")
	for _, v in T.pairs(RBM.friend_features) do
		T.table_insert(UnitPopupMenus["FRIEND"], #UnitPopupMenus["FRIEND"] - 1, v)
	end
	
	T.hooksecurefunc("UnitPopup_ShowMenu", function(dropdownMenu, which, unit, name, userData, ...)
		for i = 1, UIDROPDOWNMENU_MAXBUTTONS do
			local button = _G["DropDownList" .. UIDROPDOWNMENU_MENU_LEVEL .. "Button" .. i]
			if RBM.UnitPopupButtonsExtra[button.value] then
				button.func = UnitPopupButtons[button.value].func
			end
		end
	end)
end

local function InitializeCallback()
	if not E.db.lui.modules.misc.general["rightButtonMenu"] then return end
	RBM:Initialize()
end

E:RegisterModule(RBM:GetName(), InitializeCallback)